from flask import Blueprint, jsonify, g, redirect, url_for, render_template, request, current_app
from models.base import Session
from models.chat import Chat
from models.message import Message
from models.author import Author
from models.user import User
from models.media_download import MediaDownload
from models.user_session import UserSession
from sqlalchemy.orm import joinedload
from sqlalchemy import func, desc
from datetime import datetime, timedelta
from ..middleware.auth import auth_required
from models.enums import MediaType, DownloadStatus
from models.timestamp_tracker import TimestampTracker
import logging

logger = logging.getLogger(__name__)

bp = Blueprint('dashboard', __name__)

@bp.route("/dashboard")
@auth_required(roles=['admin', 'user'])
def dashboard():
    """Main dashboard view"""
    return render_template(
        'dashboard.html',  # Changed from 'dashboard/index.html' to 'dashboard.html'
        user=g.user,
        page_title="Dashboard"
    )

@bp.route("/api/dashboard/stats")
@auth_required(roles=['admin', 'user'])
def get_dashboard_stats():
    try:
        session = Session()
        stats = {
            "total_messages": session.query(func.count(Message.id)).scalar() or 0,
            "total_chats": session.query(func.count(Chat.id)).scalar() or 0,
            "total_authors": session.query(func.count(Author.id)).scalar() or 0,

            # Media statistics
            "total_videos": session.query(func.count(MediaDownload.id))\
                .filter(MediaDownload.media_type == MediaType.VIDEO.value).scalar() or 0,
            "total_images": session.query(func.count(MediaDownload.id))\
                .filter(MediaDownload.media_type == MediaType.IMAGE.value).scalar() or 0,

            # Download status statistics
            "downloaded_videos": session.query(func.count(MediaDownload.id))\
                .filter(
                    MediaDownload.media_type == MediaType.VIDEO.value,
                    MediaDownload.download_status == DownloadStatus.COMPLETED.value
                ).scalar() or 0,
            "downloaded_images": session.query(func.count(MediaDownload.id))\
                .filter(
                    MediaDownload.media_type == MediaType.IMAGE.value,
                    MediaDownload.download_status == DownloadStatus.COMPLETED.value
                ).scalar() or 0,

            # Failed downloads
            "failed_downloads": session.query(func.count(MediaDownload.id))\
                .filter(MediaDownload.download_status == DownloadStatus.FAILED.value)\
                .scalar() or 0,

            # Total downloads
            "total_downloads": session.query(func.count(MediaDownload.id))\
                .filter(MediaDownload.download_status == DownloadStatus.COMPLETED.value)\
                .scalar() or 0
        }

        return jsonify({
            "status": "success",
            "data": stats
        })

    except Exception as e:
        logger.error(f"Error fetching dashboard stats: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 400
    finally:
        session.close()

@bp.route("/api/dashboard/last-fetch")
@auth_required(roles=['admin', 'user'])
def get_last_fetch_time():
    try:
        session = Session()
        # Get the timestamp of the most recent message
        last_message = session.query(Message).order_by(Message.timestamp.desc()).first()

        return jsonify({
            "status": "success",
            "last_fetch": last_message.timestamp.isoformat() if last_message else None
        })

    except Exception as e:
        logger.error(f"Error getting last fetch time: {str(e)}")
        return jsonify({
            "status": "error",
            "message": "Failed to fetch last update time"
        }), 500
    finally:
        session.close()

@bp.route("/api/dashboard/table-data/<table>")
@auth_required(roles=['admin'])  # Restrict to admin only
def get_table_data(table):
    logger.info(f"Fetching table data for: {table}")
    session = Session()
    try:
        page = int(request.args.get('page', 1))
        per_page = int(request.args.get('per_page', 10))

        if table == "profile":
            query = session.query(User)
            total_count = query.count()
            users = query.offset((page - 1) * per_page)\
                        .limit(per_page)\
                        .all()

            data = [{
                'id': user.id,
                'username': user.username,
                'email': user.email,
                'full_name': user.full_name,
                'role': user.role,
                'is_active': user.is_active,
                'last_login': user.last_login.isoformat() if user.last_login else None
            } for user in users]

            return jsonify({
                "status": "success",
                "data": data,
                "pagination": {
                    "page": page,
                    "per_page": per_page,
                    "total": total_count,
                    "total_pages": (total_count + per_page - 1) // per_page
                }
            })
        elif table == "chats":
            query = session.query(Chat)
            total_count = query.count()
            chats = query.order_by(Chat.last_updated.desc())\
                        .offset((page - 1) * per_page)\
                        .limit(per_page)\
                        .all()

            data = [{
                'chat_id': chat.chat_id,
                'chat_name': chat.chat_name,
                'is_group': chat.is_group,
                'number': chat.number,
                'message_count': chat.message_count,
                'last_message': chat.last_message,
                'last_updated': chat.last_updated.isoformat() if chat.last_updated else None
            } for chat in chats]

        elif table == "messages":
            query = session.query(Message)\
                         .options(joinedload(Message.author))\
                         .options(joinedload(Message.chat))\
                         .order_by(Message.timestamp.desc())

            total_count = query.count()
            messages = query.offset((page - 1) * per_page)\
                          .limit(per_page)\
                          .all()

            data = [{
                'id': message.id,
                'caption': message.caption,
                'message_type': message.message_type,
                'author': message.author.name if message.author else None,
                'chat_name': message.chat.chat_name if message.chat else None,
                'number': message.author.number if message.author else None,
                'duplicate_count': message.duplicate_count or 0,
                'file_size': message.file_size,
                'timestamp': message.timestamp.isoformat() if message.timestamp else None
            } for message in messages]

        elif table == "authors":
            query = session.query(Author)
            total_count = query.count()
            authors = query.offset((page - 1) * per_page)\
                         .limit(per_page)\
                         .all()

            data = [author.to_dict() for author in authors]

        else:
            return jsonify({
                "status": "error",
                "message": f"Invalid table type: {table}"
            }), 400

        return jsonify({
            "status": "success",
            "data": data,
            "pagination": {
                "page": page,
                "per_page": per_page,
                "total": total_count,
                "total_pages": (total_count + per_page - 1) // per_page
            }
        })

    except Exception as e:
        logger.error(f"Error getting {table} data: {str(e)}")
        return jsonify({
            "status": "error",
            "message": f"Failed to load {table} data",
            "error": str(e)
        }), 500
    finally:
        session.close()

@bp.route("/api/dashboard/<table>/<int:record_id>", methods=['GET'])
def get_record(table, record_id):
    session = Session()
    try:
        if table == "messages":
            record = session.query(Message).get(record_id)
        elif table == "authors":
            record = session.query(Author).get(record_id)
        elif table == "chats":
            record = session.query(Chat).get(record_id)
        else:
            return jsonify({"error": "Invalid table"}), 400

        return jsonify(record.to_dict() if record else {})
    except Exception as e:
        logger.error(f"Error getting record: {str(e)}")
        return jsonify({"error": "Failed to load record"}), 500
    finally:
        session.close()

@bp.route("/api/dashboard/<table>", methods=['POST'])
def save_record(table):
    session = Session()
    try:
        data = request.json
        if table == "messages":
            record = Message(**data)
        elif table == "authors":
            record = Author(**data)
        elif table == "chats":
            record = Chat(**data)
        else:
            return jsonify({"error": "Invalid table"}), 400

        session.add(record)
        session.commit()
        return jsonify({"status": "success"})
    except Exception as e:
        session.rollback()
        logger.error(f"Error saving record: {str(e)}")
        return jsonify({"error": "Failed to save record"}), 500
    finally:
        session.close()

@bp.route("/api/dashboard/<table>/<int:record_id>", methods=['DELETE'])
def delete_record(table, record_id):
    session = Session()
    try:
        if table == "messages":
            record = session.query(Message).get(record_id)
        elif table == "authors":
            record = session.query(Author).get(record_id)
        elif table == "chats":
            record = session.query(Chat).get(record_id)
        else:
            return jsonify({"error": "Invalid table"}), 400

        if record:
            session.delete(record)
            session.commit()
            return jsonify({"status": "success"})
        return jsonify({"error": "Record not found"}), 404
    except Exception as e:
        session.rollback()
        logger.error(f"Error deleting record: {str(e)}")
        return jsonify({"error": "Failed to delete record"}), 500
    finally:
        session.close()

@bp.route("/api/dashboard/historical-stats")
def historical_stats():
    timeframe = request.args.get('timeframe', 'day')
    session = Session()
    try:
        # Calculate date range
        end_date = datetime.now()
        if timeframe == 'day':
            start_date = end_date - timedelta(days=1)
            date_format = '%Y-%m-%d %H:00:00'
        else:
            start_date = end_date - timedelta(days=7 if timeframe == 'week' else 30)
            date_format = '%Y-%m-%d'

        # Query messages with proper date formatting
        messages_query = session.query(
            func.strftime(date_format, Message.timestamp).label('date'),
            func.count(Message.id).label('count')
        ).filter(
            Message.timestamp.between(start_date, end_date)
        ).group_by('date').order_by('date')  # Changed to ascending order

        # Initialize data structure
        messages_data = {}
        current = start_date

        # Generate all possible dates/hours in the range
        while current <= end_date:
            if timeframe == 'day':
                formatted_date = current.strftime(date_format)
                current += timedelta(hours=1)
            else:
                formatted_date = current.strftime('%Y-%m-%d')
                current += timedelta(days=1)
            messages_data[formatted_date] = 0

        # Fill in actual counts
        for row in messages_query:
            if row.date in messages_data:
                messages_data[row.date] = row.count

        # Convert to list format for Chart.js
        labels = list(messages_data.keys())
        values = list(messages_data.values())

        return jsonify({
            'status': 'success',
            'labels': labels,
            'datasets': {
                'messages': values
            }
        })

    except Exception as e:
        logger.error(f"Error in historical_stats: {str(e)}")
        return jsonify({
            "status": "error",
            "error": "Internal server error",
            "message": str(e)
        }), 500
    finally:
        session.close()

@bp.route("/api/dashboard/last-timestamp")
def get_last_fetch_timestamp():
    session = Session()
    try:
        # Get the last timestamp from TimestampTracker using last_timestamp instead of timestamp
        last_timestamp = session.query(TimestampTracker)\
            .order_by(desc(TimestampTracker.last_timestamp))\
            .first()

        if last_timestamp:
            return jsonify({
                "status": "success",
                "last_timestamp": last_timestamp.last_timestamp.isoformat()
            })

        # If no timestamp found, return None
        return jsonify({
            "status": "success",
            "last_timestamp": None
        })

    except Exception as e:
        logger.error(f"Error getting last fetch timestamp: {str(e)}", exc_info=True)
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500
    finally:
        session.close()

@bp.route("/api/dashboard/<view_type>")
def get_dashboard_data(view_type):
    try:
        if view_type == "messages":
            messages = Session.query(Message).order_by(Message.timestamp.desc()).all()
            data = [{
                'id': m.id,
                'body': m.body[:100] + '...' if m.body and len(m.body) > 100 else m.body,
                'message_type': m.message_type,
                'timestamp': m.timestamp.isoformat() if m.timestamp else None,
                'chat_id': m.chat_id if hasattr(m, 'chat_id') else None,
                'author_id': m.author_id if hasattr(m, 'author_id') else None
            } for m in messages]

        elif view_type == "authors":
            authors = Session.query(Author).all()
            data = [{
                'id': a.id,
                'name': a.name,
                'number': a.number,
                'message_count': len(a.messages) if hasattr(a, 'messages') else 0
            } for a in authors]

        else:
            return jsonify({"error": "Invalid view type"}), 400

        return jsonify({"data": data})

    except Exception as e:
        print(f"Error in get_dashboard_data: {str(e)}")  # Debug log
        return jsonify({"error": str(e)}), 500

@bp.route("/api/dashboard/chats", methods=['GET'])
@auth_required(roles=['admin', 'user'])
def get_chats():
    try:
        page = int(request.args.get('page', 1))
        per_page = int(request.args.get('per_page', 20))
        search_query = request.args.get('q')

        if search_query:
            chats, total = Chat.search(search_query, page, per_page)
        else:
            chats, total = Chat.get_all(page, per_page)

        return jsonify({
            "status": "success",
            "data": [chat.to_dict() for chat in chats],
            "pagination": {
                "page": page,
                "per_page": per_page,
                "total": total,
                "total_pages": (total + per_page - 1) // per_page
            }
        })
    except Exception as e:
        logger.error(f"Error getting chats: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500

@bp.route("/api/dashboard/chats/<chat_id>", methods=['GET'])
@auth_required(roles=['admin', 'user'])
def get_chat(chat_id):
    try:
        chat = Chat.get_by_id(chat_id)
        if not chat:
            return jsonify({
                "status": "error",
                "message": "Chat not found"
            }), 404

        return jsonify({
            "status": "success",
            "data": chat.to_dict()
        })
    except Exception as e:
        logger.error(f"Error getting chat: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500

@bp.route("/api/dashboard/chats", methods=['POST'])
@auth_required(roles=['admin'])
def create_chat():
    try:
        data = request.json
        if not data:
            return jsonify({
                "status": "error",
                "message": "No data provided"
            }), 400

        # Validate required fields
        required_fields = ['chat_name']  # Only chat_name is required, number is optional
        missing_fields = [field for field in required_fields if field not in data]
        if missing_fields:
            return jsonify({
                "status": "error",
                "message": f"Missing required fields: {', '.join(missing_fields)}"
            }), 400

        # Ensure is_group is a boolean if provided
        if 'is_group' in data and not isinstance(data['is_group'], bool):
            data['is_group'] = data['is_group'] in [True, 'true', 'True', 1, '1']

        # Let the model handle the timestamps
        # The created_at and last_updated will be set automatically
        logger.info(f"Creating new chat with data: {data}")
        chat = Chat.create(data)
        return jsonify({
            "status": "success",
            "data": chat.to_dict()
        }), 201
    except Exception as e:
        logger.error(f"Error creating chat: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500

@bp.route("/api/dashboard/chats/<chat_id>", methods=['PUT'])
@auth_required(roles=['admin'])
def update_chat(chat_id):
    try:
        data = request.json
        logger.info(f"Received update request for chat {chat_id} with data: {data}")

        if not data:
            return jsonify({
                "status": "error",
                "message": "No data provided"
            }), 400

        required_fields = ['chat_name', 'is_group']  # Number is now optional
        missing_fields = [field for field in required_fields if field not in data]
        if missing_fields:
            return jsonify({
                "status": "error",
                "message": f"Missing required fields: {', '.join(missing_fields)}"
            }), 400

        # Ensure is_group is a boolean
        if 'is_group' in data and not isinstance(data['is_group'], bool):
            data['is_group'] = data['is_group'] in [True, 'true', 'True', 1, '1']

        # The last_updated field will be automatically updated by the model
        # We don't need to set it explicitly here
        logger.info(f"Processing update for chat {chat_id} with data: {data}")

        try:
            chat = Chat.update(chat_id, data)
            if not chat:
                logger.warning(f"Chat not found for update: {chat_id}")
                return jsonify({
                    "status": "error",
                    "message": "Chat not found"
                }), 404

            logger.info(f"Successfully updated chat {chat_id}")
            return jsonify({
                "status": "success",
                "data": chat.to_dict()
            })
        except Exception as update_error:
            logger.error(f"Database error updating chat: {str(update_error)}")
            return jsonify({
                "status": "error",
                "message": f"Database error: {str(update_error)}"
            }), 500
    except Exception as e:
        logger.error(f"Error in update_chat route: {str(e)}")
        return jsonify({
            "status": "error",
            "message": f"Server error: {str(e)}"
        }), 500

@bp.route("/api/dashboard/chats/<chat_id>", methods=['DELETE'])
@auth_required(roles=['admin'])
def delete_chat(chat_id):
    session = Session()
    try:
        # First try to find the chat by chat_id
        chat = session.query(Chat).filter_by(chat_id=chat_id).first()

        if not chat:
            # If not found by chat_id, try by numeric id
            try:
                chat = session.query(Chat).get(int(chat_id))
            except ValueError:
                pass

        if not chat:
            return jsonify({
                "status": "error",
                "message": f"Chat with ID {chat_id} not found"
            }), 404

        session.delete(chat)
        session.commit()

        return jsonify({
            "status": "success",
            "message": "Chat deleted successfully"
        })
    except Exception as e:
        session.rollback()
        logger.error(f"Error deleting chat: {str(e)}")
        return jsonify({
            "status": "error",
            "message": f"Failed to delete chat: {str(e)}"
        }), 500
    finally:
        session.close()

